import json

import requests
from fastapi import Depends, APIRouter, HTTPException, status
from fastapi.responses import JSONResponse

from api import get_macfinder_redis, get_conn
from util.auth import check_auth
from model.wapi.cntl import Mgr, APIToken
from model.macfinder import format_mac, MacFinderJob, OUI_DB, MacFinderJobCreationRequest, \
    MacFinderJobCreationResponse
from model.wapi.nd import VLAN
from model.settings import settings

router = APIRouter(
    prefix="/api/macfinder",
    tags=["macfinder"]
)
js_mods = ['macfinder']


@router.post('/job')
async def create_job(mac_finder_request: MacFinderJobCreationRequest,
                     conn=Depends(get_conn),
                     token: APIToken = Depends(check_auth),
                     r=Depends(get_macfinder_redis),
                     ):
    if not len(format_mac(mac_finder_request.mac)) == 17:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail='Invalid MAC'
        )

    user = Mgr.check_token(conn, token)
    vlans: list[VLAN] = VLAN.get_by_bcd(conn, user.login_name, mac_finder_request.bcd)
    vlan_valid = False

    for vlan in vlans:
        # TODO: this permission does not exist?
        if not user.has_permission(conn, 'nm.macfinder_global'):
            if user.has_bcd(conn, vlan.bcd):
                vlan_valid = True
                break
        else:
            vlan_valid = True

    if not vlan_valid:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail='Requested vlan not found!'
        )

    created_jobs = []
    for vlan in vlans:
        resp = requests.post(settings.macfinder_server_url + '/new_task',
                             cert=(settings.macfinder_cert, settings.macfinder_key),
                             verify=settings.macfinder_ca,
                             data={
                                 'mac': mac_finder_request.mac,
                                 'vlan': vlan.id,
                                 'net_instnc': vlan.net_instnc,
                                 'tag_check': 'y'
                             })
        if not resp.ok:
            raise HTTPException(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                detail='Could not create macfinder task!',
            )
        job_id = resp.json()['job_id']
        created_jobs.append(job_id)
        pipe = r.pipeline()
        pipe.multi()
        pipe.set(job_id, MacFinderJob(job_id=job_id,
                                      mac=format_mac(mac_finder_request.mac),
                                      vlan=vlan.id,
                                      bcd=mac_finder_request.bcd).model_dump_json(),
                 )
        pipe.expire(job_id, 60 * 60 * 12)
        pipe.execute()

    return MacFinderJobCreationResponse(job_ids=created_jobs)


@router.get('/job/{job_id}')
async def query_job(job_id: str,
                    conn=Depends(get_conn),
                    token: APIToken = Depends(check_auth),
                    r=Depends(get_macfinder_redis),
                    ) -> MacFinderJob | str:
    user = Mgr.check_token(conn, token)
    # return job_id
    raw_job = r.get(job_id)
    if raw_job is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail='Unknown job'
        )
    job = MacFinderJob(**json.loads(raw_job))
    if not user.has_bcd(conn, job.bcd) and not user.has_permission(conn, 'nm.macfinder_global'):
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail='Unknown job'
        )

    session = requests.sessions.Session()
    job.poll(session)
    pipe = r.pipeline()
    pipe.multi()
    pipe.set(job_id, job.model_dump_json())
    pipe.expire(job_id, 60 * 60 * 12)
    pipe.execute()
    return job


@router.get('/oui_lookup')
async def oui_lookup(mac: str):
    mac = format_mac(mac)
    vendor = OUI_DB.get(mac[:8], None)
    if mac == 'fe:ed:de:ad:be:ef':
        return JSONResponse({'mac': mac, 'vendor': 'SCC anycast gateway'})
    if vendor is None:
        return JSONResponse({'mac': mac, 'vendor': None}, status_code=status.HTTP_404_NOT_FOUND)
    if len(vendor) > 1:
        r_part = int(mac.replace(':', '')[6:], base=16)
        for v in vendor:
            if v[0] is None or v[1] is None:
                continue
            if v[0] <= r_part <= v[1]:
                return JSONResponse({'mac': mac, 'vendor': v[2]})
    return JSONResponse({'mac': mac, 'vendor': vendor[0][2]})


@router.get('/oui_dump')
async def oui_dump():
    return JSONResponse(OUI_DB)
